import os as _os
import openpyxl as _openpyxl
import pickle as _pickle
import csv as _csv
import json as _json
from binaryornot.check import is_binary as _is_binary
from ._collection import FileError
__doc__ = '''This module contains all the
functions for text file in pyil.'''

default_ele_token = [',']
default_line_token = ['\n']


def write_to_binary(obj, file):
    """using pickle.dump to convert to binary file.
    If you want to change the method, do:
    write_to_binary=your_function
    make sure your the order of the parameter of your function
    is same as this one."""
    _pickle.dump(obj, file)


class text_file:
    def __init__(self, name: str, res_name: str, next_ele_token=default_ele_token, next_line_token=default_line_token):
        """name >- the path of the original file(text file)
        res_name >- the path of the destination file
        (include file name and the extension for the first
        one but no extension for the second one)"""
        if _is_binary(name):
            raise FileError('Cannot open binary files: %s' % name)
        self._f = open(name, 'r+')
        self.content = self._f.read()
        temp, temp1, temp2 = [], '', []
        for i in self.content + next_line_token[0]:
            if i in next_ele_token:
                temp2.append(temp1)
                temp1 = ''
            elif i in next_line_token:
                temp2.append(temp1)
                temp.append(temp2)
                temp1 = ''
                temp2 = []
            else:
                temp1 += i
        self._path = res_name
        self._formatted_content = temp.copy()
        self.delete_original_file = False
        self._opath = name

    def write(self, obj: str):
        """Write to the original file."""
        self._f.write(obj)
        return len(self.content)

    def append(self, obj: str):
        """Append something to the original file."""
        self._f.write(self.content + obj)
        return len(obj)

    def convert_to_xlsx(self):
        """Convert the file into Microsoft spread sheet."""
        self._wb = _openpyxl.Workbook()
        self._sheet = self._wb.get_sheet_by_name("Sheet")
        temp = 1
        for i in self._formatted_content:
            self._write_row(i, temp)
            temp += 1

    def convert_to_binary_file(self):
        """Convert the file into binary file with extension using pickle.dump."""
        write_to_binary(self.content, open(self._path + '.dat', 'wb'))

    def convert_to_csv(self, next_cell_token=',', next_row_token='\n'):
        """convert the text file into .csv file using csv module."""
        writer = _csv.writer(open(self._path, 'w', newline=''), delimiter=next_cell_token,
                             lineterminator=next_row_token)
        for i in self._formatted_content:
            writer.writerow(i)

    def _write_row(self, list_to_write, start_row, start_column=1):
        try:
            self._sheet.cell(row=start_row, column=start_column).value = list_to_write[start_column - 1]
            return self._write_row(list_to_write, start_row, start_column=start_column + 1)
        except IndexError:
            return

    def convert_to_json(self):
        """Use json module to convert the file into .json file."""
        file = open(self._path + '.json', 'w')
        file.write(_json.dumps(self.content))
        file.close()

    def close(self):
        """Close and save the file."""
        try:
            self._wb.save(self._path + '.xlsx')
        except AttributeError:
            pass
        self._f.close()
        if self.delete_original_file:
            _os.unlink(self._opath)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()




class NoAssignedValueError(Exception):
    pass


class FileError(Exception):
    pass


def _cli(ext: str):
    raise FileError('cannot open file with this extension: %s.' % ext)


def switch_bool(v: bool):
    return False if v else True

def compare(n1:str,n2:str,i=False):
    return n1.lower()==n2.lower() if i else n1==n2

class simplify_str:
    def __init__(self, value: str, casei=False):
        self.v = value
        self.i = casei

    def simplify(self):
        try:
            self.v = float(self.v)
        except ValueError:
            pass
        try:
            self.v = int(self.v)
        except ValueError:
            pass
        try:
            self.v = self._toBool()
        except (ValueError, AttributeError):
            pass
        try:
            self.v = self._toNone()
        except (ValueError, AttributeError):
            pass
        return self.v

    def _toNone(self):
        if self.i:
            if self.v.lower() == 'none':
                return None
            raise ValueError('Unknown None value represation.')
        else:
            if self.v == 'None':
                return None
            raise ValueError('Unknown None value represation.')

    def _toBool(self):
        if self.i:
            if self.v.lower() == 'true':
                return True
            elif self.v.lower() == 'false':
                return False
            raise ValueError('Unknown boolean value.')
        else:
            if self.v == 'True':
                return True
            elif self.v == 'False':
                return False
            raise ValueError('Unknown boolean value.')




from ._text import text_file
from ._CLI import action as _action
import sys as _sys


__license__ = 'MO'
__author__ = 'G.M'
__email__ = 'G.Mpydev@gmail.com'
__quote__="""Loop and recursion work together will produce
better result and better than any of the previous."""


def cmd():
    '''call me when ever you want to enable command line control.'''
    _action(_sys.argv[1:])


def input_control():
    '''call me when ever you want to enable user input control.'''
    while True:
        temp = input('Please enter the command you want to execute(q to quit):')
        if temp == 'q':
            _sys.exit(1234567890)
        try:
            _action(temp)
        except ValueError:
            print('Invalid command.')
            continue




from ._text import *
import copy, re
from ._collection import NoAssignedValueError
from ._collection import _cli

def action(arguments: list):
    if len(arguments) <2:
        print('''Usage: <name of your original file> <name of your destination file
        without extension> <the file extension of the file you are converting to>
        [next element token, use comma to split] [next line
        token, use comma to split]''')
        raise NoAssignedValueError ( 'No value was passed.')
    argv = copy.copy(arguments)
    argv.split(' ')
    make_decision(argv, len(argv))


def which_type(obj, string: str):
    dotRe = re.compile(r'\.')
    dotRe.sub('', string)
    string = string.lower()
    return {'xlsx':            obj.convert_to_xlsx,
            'csv':             obj.convert_to_csv,
            'dat':             obj.convert_to_binary_file,
            'json':            obj.convert_to_json,
            'keywordnotfound': _cli(string)
            }.get(string, 'keywordnotfound')


def make_decision(option, howmany):
    if howmany == 5:
        with text_file(option[0], option[1], next_ele_token=option[3], next_line_token=option[4]) as f:
            which_type(f, option[2])()
    elif howmany == 4:
        with text_file(option[0], option[1], next_ele_token=option[3]) as f:
            which_type(f, option[2])()
    elif howmany == 3:
        with text_file(option[0], option[1]) as f:
            which_type(f, option[2])()



from setuptools import setup
import re
from re import DOTALL

v = '0.2.7.5'


def what():
    with open('''update.log''')as file:
        temp = file.read()
        tempre = re.compile('v' + v + '.+', flags=DOTALL)
        tempm = tempre.search(temp)
        print('\n\n', tempm.group(0), '\n\n')


def read(name):
    with open(name) as file:
        return file.read()


what()
try:
    setup(
        name='pyil-converter',
        version=v,
        packages=['pyil'],
        license='MIT',
        author='G.M',
        author_email='G.Mpydev@gmail.com',
        description='Convert files to the format you want!',
        long_description=read('README.rst'),
        install_requires=[
            'openpyxl', 'binaryornot'],
        include_package_data=True,
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Programming Language :: Python :: 3 :: Only',
            'Operating System :: Microsoft :: Windows',
            'Intended Audience :: Developers',
            'Intended Audience :: Information Technology',
            'Natural Language :: English',
            'Topic :: System :: Filesystems',
            'Topic :: System :: Archiving :: Packaging',
            'Topic :: Utilities',
            'License :: OSI Approved :: MIT License'
        ],
        platforms='win32',
        keywords=['file', 'format', 'extension', 'reformatting',
                                            'converting','data'],
        url=r'https://wasted123.github.io/pyil/'
    )
    print('Installation succeeded.')

except Exception as msg:
    print('\n')
    print('An unexpected exception happened.')
    print('\n')
    print(str(msg))
    print('\n')
    print('If it is because installation error, fro example '
          ' when compiling c extensions, an error occurred, I '
          'personally recommend try install the python wheel '
          'for the nodule you want to install.''')





import os

from ._collection import simplify_str
import os as _os

true = True
false = False


def search(name, *path, **option):
    '''Search the given file name in the given path, if path
    is none, will search in C:\\(It will search in the order
    you let it to). If enable list_all, it will return a
    generator containing all result.'''
    from ._collection import compare
    if len(path) == 0:
        path = 'C:\\'
    I, topDown, list_all = false, true, false
    for key, value in option.items():
        if key == 'caseI':
            I = option['caseI']
        elif key == 'topdown':
            topDown = option['topdown']
        elif key == 'list_all':
            list_all = option[list_all]
    for i in path:
        for current_folder, subfolder, subfile in os.walk(i, topdown=topDown):
            for fname in subfile:
                if compare(fname, name, i=I):
                    if list_all:
                        yield _os.path.abspath(os.path.join(current_folder, fname))
                    else:
                        return _os.path.abspath(os.path.join(current_folder, fname))
    return None


def parselist(src: str, caseI=True):
    '''Parse a string into list: '[1,2,3]'>-[1,2,3].
    If their is some problem with the string containing
    the list, it would probably give an incorrect result.'''
    from ._collection import switch_bool

    src = src[1:-1] + ','
    in_quote, reading = false, false
    part = ''
    final = []

    for i in src:
        if in_quote == false and i == ']':
            part += i
            part = (parselist(part))
            reading = false

        elif (in_quote == false and i == '[') or reading == true:
            reading = true
            part += i
            continue
        elif i == '\'':
            try:
                if i[src.index(i) + 1] == ',' or i[src.index(i) - 1] == ',':
                    in_quote = switch_bool(in_quote)
            except IndexError:
                pass

        elif i == ',' and in_quote == false:
            final.append(part)
            part = ''

        elif in_quote:
            part += i

    return final


print(search('a.tXt.txt', caseI=True))
